# -*- coding: utf-8 -*-
__author__ = 'Ardy Dedase'
__email__ = 'ardy.dedase@skyscanner.net'

__copyright__ = "Copyright (C) 2016 Skyscanner Ltd"
__license__ = """
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the specific
language governing permissions and limitations under the License.
"""

from .skyscanner import Flights, FlightsCache, Hotels, CarHire



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Authors: ardydedase <ardy.dedase@skyscanner.net>
# @Date:   2015-10-05 22:31:56
# @Last Modified by:   ardydedase
# @Last Modified time: 2016-04-25 11:17:49

__credits__ = ["Ardy Dedase", "Denis Dudnik"]
__copyright__ = "Copyright (C) 2016 Skyscanner Ltd"
__license__ = """
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the specific
language governing permissions and limitations under the License.
"""

import sys
import time
import logging
import requests
try:
    import lxml.etree as etree
except ImportError:
    import xml.etree.ElementTree as etree


def configure_logger(log_level=logging.WARN):
    logger = logging.getLogger(__name__)
    logger.setLevel(log_level)
    try:
        sa = logging.StreamHandler(stream=sys.stdout)
    except TypeError:
        sa = logging.StreamHandler()
    formatter = logging.Formatter(
        '%(asctime)s - %(filename)s:%(lineno)d - %(levelname)s - %(message)s')
    sa.setFormatter(formatter)
    logger.addHandler(sa)
    return logger

log = configure_logger()
STRICT, GRACEFUL, IGNORE = 'strict', 'graceful', 'ignore'


class ExceededRetries(Exception):

    """Is thrown when allowed number of polls were
    performed but response is not complete yet."""
    pass


class EmptyResponse(Exception):

    """Is thrown when API returns empty response."""
    pass


class MissingParameter(KeyError):

    """Is thrown when expected request parameter is missing."""
    pass


class Transport(object):

    """
    Parent class for initialization
    """
    API_HOST = 'http://partners.api.skyscanner.net'
    MARKET_SERVICE_URL = '{api_host}/apiservices/reference/v1.0/countries'.format(
        api_host=API_HOST)
    LOCATION_AUTOSUGGEST_URL = '{api_host}/apiservices/autosuggest/v1.0'.format(
        api_host=API_HOST)
    LOCATION_AUTOSUGGEST_PARAMS = ('market', 'currency', 'locale')
    _SUPPORTED_FORMATS = ('json', 'xml')

    def __init__(self, api_key, response_format='json'):
        """
        :param api_key - The API key to identify ourselves
        :param response_format - specify preferred format of the response, default is 'json'
        """
        if not api_key:
            raise ValueError('API key must be specified.')
        if response_format.lower() not in self._SUPPORTED_FORMATS:
            raise ValueError('Unknown response format: %s, supported formats are: %s' %
                             (response_format, ', '.join(self._SUPPORTED_FORMATS)))
        self.api_key = api_key
        self.response_format = response_format.lower()

    def get_additional_params(self, **params):
        """
        Filter to get the additional params needed for polling
        """

        # TODO: Move these params to their own vertical if needed.
        polling_params = [
            'locationschema',
            'carrierschema',
            'sorttype',
            'sortorder',
            'originairports',
            'destinationairports',
            'stops',
            'outbounddeparttime',
            'outbounddepartstarttime',
            'outbounddepartendtime',
            'inbounddeparttime',
            'inbounddepartstarttime',
            'inbounddepartendtime',
            'duration',
            'includecarriers',
            'excludecarriers'
        ]

        additional_params = dict(
            (key, value) for key, value in params.items() if key in polling_params)

        return additional_params

    def get_result(self, errors=STRICT, **params):
        """
        Get all results, no filtering, etc. by creating and polling the session.
        """
        additional_params = self.get_additional_params(**params)
        return self.poll_session(self.create_session(**params), errors=errors, **additional_params)

    def make_request(self, service_url, method='get', headers=None, data=None,
                     callback=None, errors=STRICT, **params):
        """
        Reusable method for performing requests.

        :param service_url - URL to request
        :param method - request method, default is 'get'
        :param headers - request headers
        :param data - post data
        :param callback - callback to be applied to response,
                          default callback will parse response as json object.
        :param errors - specifies communication errors handling mode, possible values are:
                         * strict (default) - throw an error as soon as one occurred
                         * graceful - ignore certain errors, e.g. EmptyResponse
                         * ignore - ignore all errors and return a result in any case.
                                    NOTE that it DOES NOT mean that no exceptions can be
                                    raised from this method, it mostly ignores communication
                                    related errors.
                         * None or empty string equals to default
        :param params - additional query parameters for request
        """
        error_modes = (STRICT, GRACEFUL, IGNORE)
        error_mode = errors or GRACEFUL
        if error_mode.lower() not in error_modes:
            raise ValueError(
                'Possible values for errors argument are: %s' % ', '.join(error_modes))

        if callback is None:
            callback = self._default_resp_callback

        if 'apikey' not in service_url.lower():
            params.update({
                'apiKey': self.api_key
            })

        request = getattr(requests, method.lower())

        log.debug('* Request URL: %s' % service_url)
        log.debug('* Request method: %s' % method)
        log.debug('* Request query params: %s' % params)
        log.debug('* Request headers: %s' % headers)

        r = request(service_url, headers=headers, data=data, params=params)
        try:
            r.raise_for_status()
            return callback(r)
        except Exception as e:
            return self._with_error_handling(r, e, error_mode, self.response_format)

    def get_markets(self, market):
        """
        Get the list of markets
        http://business.skyscanner.net/portal/en-GB/Documentation/Markets
        """
        url = "{url}/{market}".format(url=self.MARKET_SERVICE_URL,
                                      market=market)
        return self.make_request(url, headers=self._headers())

    def location_autosuggest(self, **params):
        """
        Location Autosuggest Services
        Doc URLs: http://business.skyscanner.net/portal/en-GB/Documentation/Autosuggest
                  http://business.skyscanner.net/portal/en-GB/Documentation/CarHireAutoSuggest
                  http://business.skyscanner.net/portal/en-GB/Documentation/HotelsAutoSuggest
        Format: Generic -        {LOCATION_AUTOSUGGEST_URL}/{market}/{currency}/{locale}/?query={query}&apiKey={apiKey}
                CarHire/Hotels - {LOCATION_AUTOSUGGEST_URL}/{market}/{currency}/{locale}/{query}?apiKey={apiKey}
        """
        service_url = "{url}/{params_path}".format(
            url=self.LOCATION_AUTOSUGGEST_URL,
            params_path=self._construct_params(
                params, self.LOCATION_AUTOSUGGEST_PARAMS)
        )
        return self.make_request(service_url, headers=self._headers(), **params)

    def create_session(self, **params):
        """Creates a session for polling. Should be implemented by sub-classes"""
        raise NotImplementedError('Should be implemented by a sub-class.')

    def poll_session(self, poll_url, initial_delay=2, delay=1, tries=20, errors=STRICT, **params):
        """
        Poll the URL
        :param poll_url - URL to poll, should be returned by 'create_session' call
        :param initial_delay - specifies how many seconds to wait before the first poll
        :param delay - specifies how many seconds to wait between the polls
        :param tries - number of polls to perform
        :param errors - errors handling mode, see corresponding parameter in 'make_request' method
        :param params - additional query params for each poll request
        """
        time.sleep(initial_delay)
        poll_response = None
        for n in range(tries):
            poll_response = self.make_request(poll_url, headers=self._headers(),
                                              errors=errors, **params)

            if self.is_poll_complete(poll_response):
                return poll_response
            else:
                time.sleep(delay)

        if STRICT == errors:
            raise ExceededRetries(
                "Failed to poll within {0} tries.".format(tries))
        else:
            return poll_response

    def is_poll_complete(self, poll_resp):
        """
        Checks the condition in poll response to determine if it is complete
        and no subsequent poll requests should be done.
        """
        if poll_resp.parsed is None:
            return False
        success_list = ['UpdatesComplete', True, 'COMPLETE']
        status = None
        if self.response_format == 'xml':
            status = poll_resp.parsed.find('./Status').text
        elif self.response_format == 'json':
            status = poll_resp.parsed.get(
                'Status', poll_resp.parsed.get('status'))
        if status is None:
            raise RuntimeError('Unable to get poll response status.')
        return status in success_list

    @staticmethod
    def _with_error_handling(resp, error, mode, response_format):

        def safe_parse(r):
            try:
                return Transport._parse_resp(r, response_format)
            except (ValueError, SyntaxError) as ex:
                log.error(ex)
                r.parsed = None
                return r

        if isinstance(error, requests.HTTPError):
            if resp.status_code == 400:
                # It means that request parameters were rejected by the server,
                # so we need to enrich standard error message with 'ValidationErrors'
                # from the response
                resp = safe_parse(resp)
                if resp.parsed is not None:
                    parsed_resp = resp.parsed
                    messages = []
                    if response_format == 'xml' and parsed_resp.find('./ValidationErrors') is not None:
                        messages = [e.find('./Message').text
                                    for e in parsed_resp.findall('./ValidationErrors/ValidationErrorDto')]
                    elif response_format == 'json' and 'ValidationErrors' in parsed_resp:
                        messages = [e['Message']
                                    for e in parsed_resp['ValidationErrors']]
                    error = requests.HTTPError(
                        '%s: %s' % (error, '\n\t'.join(messages)), response=resp)
            elif resp.status_code == 429:
                error = requests.HTTPError('%sToo many requests in the last minute.' % error,
                                           response=resp)

        if STRICT == mode:
            raise error
        elif GRACEFUL == mode:
            if isinstance(error, EmptyResponse):
                # Empty response is returned by the API occasionally,
                # in this case it makes sense to ignore it and retry.
                log.warning(error)
                resp.parsed = None
                return resp

            elif isinstance(error, requests.HTTPError):
                # Ignoring 'Too many requests' error,
                # since subsequent retries will come after a delay.
                if resp.status_code == 429:    # Too many requests
                    log.warning(error)
                    return safe_parse(resp)
                else:
                    raise error
            else:
                raise error
        else:
            # ignore everything, just log it and return whatever response we
            # have
            log.error(error)
            return safe_parse(resp)

    def _session_headers(self):
        headers = self._headers()
        headers.update({'Content-Type': 'application/x-www-form-urlencoded'})
        return headers

    def _headers(self):
        return {'Accept': 'application/%s' % self.response_format}

    def _default_resp_callback(self, resp):
        if not resp or not resp.content:
            raise EmptyResponse('Response has no content.')

        try:
            parsed_resp = self._parse_resp(resp, self.response_format)
        except (ValueError, SyntaxError):
            raise ValueError('Invalid %s in response: %s...' %
                             (self.response_format.upper(), resp.content[:100]))

        return parsed_resp

    @staticmethod
    def _construct_params(params, required_keys, opt_keys=None):
        """
        Construct params list in order of given keys.
        """
        try:
            params_list = [params.pop(key) for key in required_keys]
        except KeyError as e:
            raise MissingParameter(
                'Missing expected request parameter: %s' % e)
        if opt_keys:
            params_list.extend([params.pop(key)
                                for key in opt_keys if key in params])
        return '/'.join(str(p) for p in params_list)

    @staticmethod
    def _parse_resp(resp, response_format):
        resp.parsed = etree.fromstring(
            resp.content) if response_format == 'xml' else resp.json()
        return resp


class Flights(Transport):

    """
    Flights Live Pricing
    http://business.skyscanner.net/portal/en-GB/Documentation/FlightsLivePricingList
    """

    PRICING_SESSION_URL = '{api_host}/apiservices/pricing/v1.0'.format(
        api_host=Transport.API_HOST)

    def create_session(self, **params):
        """
        Create the session
        date format: YYYY-mm-dd
        location: ISO code
        """
        return self.make_request(self.PRICING_SESSION_URL,
                                 method='post',
                                 headers=self._session_headers(),
                                 callback=lambda resp: resp.headers[
                                     'location'],
                                 data=params)

    def request_booking_details(self, poll_url, **params):
        """
        Request for booking details
        URL Format: {API_HOST}/apiservices/pricing/v1.0/{session key}/booking?apiKey={apiKey}
        """
        return self.make_request("%s/booking" % poll_url,
                                 method='put',
                                 headers=self._headers(),
                                 callback=lambda resp: resp.headers[
                                     'location'],
                                 **params)


class FlightsCache(Flights):

    """
    Flights Browse Cache
    http://business.skyscanner.net/portal/en-GB/Documentation/FlightsBrowseCacheOverview
    """

    BROWSE_QUOTES_SERVICE_URL = '{api_host}/apiservices/browsequotes/v1.0'.format(
        api_host=Transport.API_HOST)
    BROWSE_ROUTES_SERVICE_URL = '{api_host}/apiservices/browseroutes/v1.0'.format(
        api_host=Transport.API_HOST)
    BROWSE_DATES_SERVICE_URL = '{api_host}/apiservices/browsedates/v1.0'.format(
        api_host=Transport.API_HOST)
    BROWSE_GRID_SERVICE_URL = '{api_host}/apiservices/browsegrid/v1.0'.format(
        api_host=Transport.API_HOST)
    _REQ_PARAMS = ('market', 'currency', 'locale',
                   'originplace', 'destinationplace', 'outbounddate')
    _OPT_PARAMS = ('inbounddate',)

    def get_cheapest_price_by_date(self, **params):
        """
        {API_HOST}/apiservices/browsedates/v1.0/{market}/{currency}/{locale}/{originPlace}/{destinationPlace}/{outboundPartialDate}/{inboundPartialDate}?apiKey={apiKey}
        """
        service_url = "{url}/{params_path}".format(
            url=self.BROWSE_DATES_SERVICE_URL,
            params_path=self._construct_params(
                params, self._REQ_PARAMS, self._OPT_PARAMS)
        )

        return self.make_request(service_url, headers=self._headers(), **params)

    def get_cheapest_price_by_route(self, **params):
        """
        {API_HOST}/apiservices/browseroutes/v1.0/{market}/{currency}/{locale}/{originPlace}/{destinationPlace}/{outboundPartialDate}/{inboundPartialDate}?apiKey={apiKey}
        """
        service_url = "{url}/{params_path}".format(
            url=self.BROWSE_ROUTES_SERVICE_URL,
            params_path=self._construct_params(
                params, self._REQ_PARAMS, self._OPT_PARAMS)

        )
        return self.make_request(service_url, headers=self._headers(), **params)

    def get_cheapest_quotes(self, **params):
        """
        {API_HOST}/apiservices/browsequotes/v1.0/{market}/{currency}/{locale}/{originPlace}/{destinationPlace}/{outboundPartialDate}/{inboundPartialDate}?apiKey={apiKey}
        """
        service_url = "{url}/{params_path}".format(
            url=self.BROWSE_QUOTES_SERVICE_URL,
            params_path=self._construct_params(
                params, self._REQ_PARAMS, self._OPT_PARAMS)
        )
        return self.make_request(service_url, headers=self._headers(), **params)

    def get_grid_prices_by_date(self, **params):
        """
        {API_HOST}/apiservices/browsegrid/v1.0/{market}/{currency}/{locale}/{originPlace}/{destinationPlace}/{outboundPartialDate}/{inboundPartialDate}?apiKey={apiKey}
        """
        service_url = "{url}/{params_path}".format(
            url=self.BROWSE_GRID_SERVICE_URL,
            params_path=self._construct_params(
                params, self._REQ_PARAMS, self._OPT_PARAMS)
        )
        return self.make_request(service_url, headers=self._headers(), **params)


class CarHire(Transport):

    """
    Carhire Live Pricing
    {API_HOST}/apiservices/carhire/liveprices/v2/{market}/{currency}/{locale}/{pickupplace}/{dropoffplace}/{pickupdatetime}/{dropoffdatetime}/{driverage}?apiKey={apiKey}&userip={userip}
    """

    PRICING_SESSION_URL = '{api_host}/apiservices/carhire/liveprices/v2'.format(
        api_host=Transport.API_HOST)
    LOCATION_AUTOSUGGEST_URL = '{api_host}/apiservices/hotels/autosuggest/v2'.format(
        api_host=Transport.API_HOST)
    LOCATION_AUTOSUGGEST_PARAMS = ('market', 'currency', 'locale', 'query')

    def create_session(self, **params):
        """
        Create the session
        date format: YYYY-MM-DDThh:mm
        location: ISO code
        """

        service_url = "{url}/{params_path}".format(
            url=self.PRICING_SESSION_URL,
            params_path=self._construct_params(params, ('market', 'currency', 'locale',
                                                        'pickupplace', 'dropoffplace',
                                                        'pickupdatetime', 'dropoffdatetime',
                                                        'driverage'))
        )

        poll_path = self.make_request(service_url,
                                      headers=self._session_headers(),
                                      callback=lambda resp: resp.headers[
                                          'location'],
                                      userip=params['userip'])

        return "{url}{path}".format(url=self.API_HOST, path=poll_path)

    def is_poll_complete(self, poll_resp):
        if poll_resp.parsed is None:
            return False
        websites = None
        if self.response_format == 'xml':
            websites = poll_resp.parsed.findall('./Websites/WebsiteDto')
        elif self.response_format == 'json':
            websites = poll_resp.parsed['websites']
        if len(websites) == 0:
            return False
        return all(not bool(w.get('in_progress')) for w in websites)


class Hotels(Transport):

    """
    Hotels Live prices

    {API_HOST}/apiservices/hotels/liveprices/v2/{market}/{currency}/{locale}/{entityid}/{checkindate}/{checkoutdate}/{guests}/{rooms}?apiKey={apiKey}[&pageSize={pageSize}][&imageLimit={imageLimit}]
    """

    PRICING_SESSION_URL = '{api_host}/apiservices/hotels/liveprices/v2'.format(
        api_host=Transport.API_HOST)
    LOCATION_AUTOSUGGEST_URL = '{api_host}/apiservices/hotels/autosuggest/v2'.format(
        api_host=Transport.API_HOST)
    LOCATION_AUTOSUGGEST_PARAMS = ('market', 'currency', 'locale', 'query')

    def create_session(self, **params):
        """
        Create the session
        date format: YYYY-MM-DDThh:mm
        location: ISO code
        """

        service_url = "{url}/{params_path}".format(
            url=self.PRICING_SESSION_URL,
            params_path=self._construct_params(params, ('market', 'currency', 'locale',
                                                        'entityid', 'checkindate', 'checkoutdate',
                                                        'guests', 'rooms'))
        )

        poll_path = self.make_request(service_url,
                                      headers=self._session_headers(),
                                      callback=lambda resp: resp.headers['location'])

        return "{url}{path}".format(url=self.API_HOST, path=poll_path)



# @Author: ardydedase <ardy.dedase@skyscanner.net>
# @Date:   2016-03-12 19:47:15
# @Last Modified by:   ardydedase
# @Last Modified time: 2016-04-25 11:15:49
#!/usr/bin/env python
# -*- coding: utf-8 -*-

__credits__ = ["Ardy Dedase", "Denis Dudnik"]
__copyright__ = "Copyright (C) 2016 Skyscanner Ltd"
__license__ = """
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the specific
language governing permissions and limitations under the License.
"""

"""
test_skyscanner
----------------------------------

Tests for `skyscanner` module.
"""

import unittest
import json
import pprint
from datetime import datetime, timedelta
from requests import HTTPError

from skyscanner.skyscanner import (Flights, Transport, FlightsCache,
                                   CarHire, Hotels,
                                   EmptyResponse, MissingParameter,
                                   STRICT, GRACEFUL, IGNORE)

# TODO: Mock responses


class SkyScannerTestCase(unittest.TestCase):

    """Generic TestCase class to support default failure messages."""

    def setUp(self):
        # This API Key is meant for tests only
        self.api_key = 'py495888586774232134437415165965'

        self.result = None

    def tearDown(self):
        self.result = None

    def assertTrue(self, expr, msg=None):
        super(SkyScannerTestCase, self).assertTrue(
            expr, msg=msg or self._default_message())

    def _default_message(self):
        return ('API Response: %s...' % (str(self.result)[:100])) if self.result is not None else None


class FakeResponse(object):

    def __init__(self, status_code=200, content=None):
        self.content = content or ''
        self.status_code = status_code

    def json(self):
        return json.loads(self.content)


class TestTransport(SkyScannerTestCase):

    def test_get_markets_json(self):
        transport = Transport(self.api_key, response_format='json')
        self.result = transport.get_markets('en-GB').parsed

        self.assertTrue('Countries' in self.result)
        self.assertTrue(len(self.result['Countries']) > 0)

    def test_get_markets_xml(self):
        transport = Transport(self.api_key, response_format='xml')
        self.result = transport.get_markets('de-DE').parsed

        self.assertTrue(None != self.result.find('./Countries'))
        self.assertTrue(len(self.result.findall('./Countries/CountryDto')) > 0)

    def test_location_autosuggest_json(self):
        transport = Transport(self.api_key, response_format='json')
        self.result = transport.location_autosuggest(query='KUL',
                                                     market='UK',
                                                     currency='GBP',
                                                     locale='en-GB').parsed

        self.assertTrue('Places' in self.result)
        self.assertTrue(len(self.result['Places']) > 0)

    def test_location_autosuggest_xml(self):
        transport = Transport(self.api_key, response_format='xml')
        self.result = transport.location_autosuggest(query='BER',
                                                     market='DE',
                                                     currency='EUR',
                                                     locale='de-DE').parsed

        self.assertTrue(None != self.result.find('./Places'))
        self.assertTrue(len(self.result.findall('./Places/PlaceDto')) > 0)

    def test_create_session(self):
        self.assertRaises(NotImplementedError,
                          Transport(self.api_key).create_session)

    def test_with_error_handling_strict(self):
        for format in ['json', 'xml']:
            try:
                self.assertRaises(RuntimeError,
                                  Transport._with_error_handling, FakeResponse(), RuntimeError, STRICT, format)

                self.assertRaises(HTTPError,
                                  Transport._with_error_handling, FakeResponse(status_code=404), HTTPError(), STRICT, format)

                self.assertRaises(HTTPError,
                                  Transport._with_error_handling, FakeResponse(status_code=429), HTTPError('429: '), STRICT, format)
                try:
                    Transport._with_error_handling(
                        FakeResponse(status_code=429), HTTPError('429: '), STRICT, format)
                except HTTPError as e:
                    self.assertEqual(
                        e.message, '429: Too many requests in the last minute.')

                self.assertRaises(HTTPError,
                                  Transport._with_error_handling, FakeResponse(status_code=400), HTTPError('400'), STRICT, format)
                try:
                    Transport._with_error_handling(
                        FakeResponse(status_code=400), HTTPError('400'), STRICT, format)
                except HTTPError as e:
                    self.assertEqual(e.message, '400')
            except Exception as e:
                # Exception for Python 2.6
                print(e)

        self.assertRaises(HTTPError,
                          Transport._with_error_handling, FakeResponse(status_code=400,
                                                                       content='{"ValidationErrors": '
                                                                       '[{"Message": "1"}, {"Message": "2"}]}'),
                          HTTPError('400'), STRICT, 'json')
        try:
            Transport._with_error_handling(FakeResponse(status_code=400,
                                                        content='{"ValidationErrors": '
                                                                '[{"Message": "1"}, {"Message": "2"}]}'),
                                           HTTPError('400'), STRICT, 'json')
        except HTTPError as e:
            try:
                self.assertEqual(
                    e.message, ('400: %s' % '\n\t'.join(['1', '2'])))
            except AttributeError as e:
                # Exception for Python 3
                print(e)

        try:
            self.assertRaises(HTTPError,
                              Transport._with_error_handling, FakeResponse(status_code=400,
                                                                           content='<Root><ValidationErrors>'
                                                                           '<ValidationErrorDto>'
                                                                           '<Message>1</Message'
                                                                           '</ValidationErrorDto>'
                                                                           '<ValidationErrorDto>'
                                                                           '<Message>2</Message'
                                                                           '</ValidationErrorDto>'
                                                                           '</ValidationErrors></Root>'),
                              HTTPError('400'), STRICT, 'xml')
        except Exception as e:
            # Exception for Python 2.6
            print(e)

        try:
            try:
                Transport._with_error_handling(FakeResponse(status_code=400,
                                                            content='<Root><ValidationErrors>'
                                                                    '<ValidationErrorDto>'
                                                                    '<Message>1</Message'
                                                                    '</ValidationErrorDto>'
                                                                    '<ValidationErrorDto>'
                                                                    '<Message>2</Message'
                                                                    '</ValidationErrorDto>'
                                                                    '</ValidationErrors></Root>'),
                                               HTTPError('400'), STRICT, 'xml')
            except Exception as e:
                # Exception for Python 2.6
                print(e)
        except HTTPError as e:
            try:
                self.assertEqual(e.message, '400')
            except AttributeError as e:
                # Exception for Python 3
                print(e)

    def test_with_error_handling_graceful(self):
        result = Transport._with_error_handling(
            FakeResponse(), EmptyResponse(), GRACEFUL, 'json')
        self.assertTrue(None != result)
        self.assertEqual(None, result.parsed)
        result = Transport._with_error_handling(
            FakeResponse(), EmptyResponse(), GRACEFUL, 'xml')
        self.assertTrue(None != result)
        self.assertEqual(None, result.parsed)

        result = Transport._with_error_handling(FakeResponse(content='{"valid": 1}', status_code=429),
                                                HTTPError(), GRACEFUL, 'json').parsed
        self.assertTrue(None != result)
        self.assertTrue('valid' in result)
        self.assertEqual(result['valid'], 1)
        result = Transport._with_error_handling(FakeResponse(content='<valid>1</valid>', status_code=429),
                                                HTTPError(), GRACEFUL, 'xml').parsed
        self.assertTrue(None != result)
        self.assertEqual(result.tag, 'valid')
        self.assertEqual(result.text, '1')

        result = Transport._with_error_handling(FakeResponse(content='invalid', status_code=429),
                                                HTTPError(), GRACEFUL, 'json').parsed
        self.assertEqual(None, result)

        try:
            result = Transport._with_error_handling(FakeResponse(content='invalid', status_code=429),
                                                    HTTPError(), GRACEFUL, 'xml').parsed
        except Exception as e:
            print(e)

        self.assertEqual(None, result)

        for format in ['json', 'xml']:
            self.assertRaises(HTTPError,
                              Transport._with_error_handling,
                              FakeResponse(), HTTPError(), GRACEFUL, format)
            self.assertRaises(RuntimeError,
                              Transport._with_error_handling,
                              FakeResponse(), RuntimeError(), GRACEFUL, format)

    def test_with_error_handling_ignore(self):
        result = Transport._with_error_handling(
            FakeResponse(), EmptyResponse(), IGNORE, 'json').parsed
        self.assertEqual(None, result)
        result = Transport._with_error_handling(
            FakeResponse(), RuntimeError(), IGNORE, 'json').parsed
        self.assertEqual(None, result)
        result = Transport._with_error_handling(
            FakeResponse(), HTTPError(), IGNORE, 'json').parsed
        self.assertEqual(None, result)

        try:
            result = Transport._with_error_handling(
                FakeResponse(), EmptyResponse(), IGNORE, 'xml').parsed
            self.assertEqual(None, result)
            result = Transport._with_error_handling(
                FakeResponse(), RuntimeError(), IGNORE, 'xml').parsed
            self.assertEqual(None, result)
            result = Transport._with_error_handling(
                FakeResponse(), HTTPError(), IGNORE, 'xml').parsed
            self.assertEqual(None, result)
        except Exception as e:
            # Exception for Python 2.6
            print(e)

        result = Transport._with_error_handling(
            FakeResponse(content='{"valid": 1}'), HTTPError(), IGNORE, 'json').parsed
        self.assertTrue(None != result)
        self.assertTrue('valid' in result)
        self.assertEqual(result['valid'], 1)
        result = Transport._with_error_handling(
            FakeResponse(content='<valid>1</valid>'), HTTPError(), IGNORE, 'xml').parsed
        self.assertTrue(None != result)
        self.assertEqual(result.tag, 'valid')
        self.assertEqual(result.text, '1')

    def test_default_resp_callback_json(self):
        t = Transport(self.api_key, response_format='json')
        self.assertRaises(EmptyResponse,
                          t._default_resp_callback, None)
        self.assertRaises(EmptyResponse,
                          t._default_resp_callback, FakeResponse(content=''))

        self.assertRaises(ValueError,
                          t._default_resp_callback, FakeResponse(content='invalid json'))

        resp_json = t._default_resp_callback(
            FakeResponse(content='{"valid": 1}')).parsed
        self.assertTrue(None != resp_json)
        self.assertTrue('valid' in resp_json)
        self.assertEqual(resp_json['valid'], 1)

    def test_default_resp_callback_xml(self):
        t = Transport(self.api_key, response_format='xml')
        self.assertRaises(EmptyResponse,
                          t._default_resp_callback, None)
        self.assertRaises(EmptyResponse,
                          t._default_resp_callback, FakeResponse(content=''))

        try:
            self.assertRaises(ValueError,
                              t._default_resp_callback, FakeResponse(content='invalid XML'))
        except:
            self.assertRaises(Exception,
                              t._default_resp_callback, FakeResponse(content='invalid XML'))

        resp_xml = t._default_resp_callback(
            FakeResponse(content='<valid a="test">1</valid>')).parsed
        self.assertTrue(None != resp_xml)
        self.assertEqual(resp_xml.tag, 'valid')
        self.assertEqual(resp_xml.text, '1')
        self.assertEqual(resp_xml.get('a'), 'test')

    def test_construct_params(self):
        params = dict(a=1, b=2, c=3)
        self.assertEqual(
            '1/2/3', Transport._construct_params(params, ('a', 'b', 'c')))
        params = dict(a=1, b=2)
        self.assertEqual(
            '1/2', Transport._construct_params(params, ('a', 'b'), ('c',)))
        params = dict(a=1, b=2, c=3)
        self.assertEqual(
            '1/2/3', Transport._construct_params(params, ('a', 'b'), ('c',)))

        params = dict(a=1, c=3)
        self.assertRaises(MissingParameter,
                          Transport._construct_params, params, ('a', 'b'), ('c',))


class TestCarHire(SkyScannerTestCase):

    def setUp(self):
        super(TestCarHire, self).setUp()
        datetime_format = '%Y-%m-%dT%H:%S'
        pickup_datetime = datetime.now() + timedelta(days=10)
        dropoff_datetime = pickup_datetime + timedelta(days=3)
        self.pickup = pickup_datetime.strftime(datetime_format)
        self.dropoff = dropoff_datetime.strftime(datetime_format)

    def test_location_autosuggest_json(self):
        carhire_service = CarHire(self.api_key, response_format='json')

        self.result = carhire_service.location_autosuggest(
            market='UK',
            currency='GBP',
            locale='en-GB',
            query='Kuala').parsed

        self.assertTrue('results' in self.result)
        self.assertTrue(len(self.result['results']) > 0)

    def test_location_autosuggest_xml(self):
        carhire_service = CarHire(self.api_key, response_format='xml')

        self.result = carhire_service.location_autosuggest(
            market='DE',
            currency='EUR',
            locale='de-DE',
            query='Berlin').parsed

        self.assertTrue(None != self.result.find('./Results'))
        self.assertTrue(
            len(self.result.findall('./Results/HotelResultDto')) > 0)

    def test_get_result_json(self):
        """
        http://partners.api.skyscanner.net/apiservices/carhire/liveprices/v2/{market}/{currency}/{locale}/{pickupplace}/{dropoffplace}/{pickupdatetime}/{dropoffdatetime}/{driverage}?apiKey={apiKey}&userip={userip}
        YYYY-MM-DDThh:mm
        """
        carhire_service = CarHire(self.api_key, response_format='json')

        self.result = carhire_service.get_result(
            market='UK',
            currency='GBP',
            locale='en-GB',
            pickupplace='LHR-sky',
            dropoffplace='LHR-sky',
            pickupdatetime=self.pickup,
            dropoffdatetime=self.dropoff,
            driverage='30',
            userip='175.156.244.174').parsed

        self.assertTrue('cars' in self.result)
        self.assertTrue('websites' in self.result)

    # def test_get_result_xml(self):
    #     """
    #     http://partners.api.skyscanner.net/apiservices/carhire/liveprices/v2/{market}/{currency}/{locale}/{pickupplace}/{dropoffplace}/{pickupdatetime}/{dropoffdatetime}/{driverage}?apiKey={apiKey}&userip={userip}
    #     YYYY-MM-DDThh:mm
    #     """
    #     carhire_service = CarHire(self.api_key, response_format='xml')

    #     self.result = carhire_service.get_result(
    #         market='UK',
    #         currency='GBP',
    #         locale='en-GB',
    #         pickupplace='LHR-sky',
    #         dropoffplace='LHR-sky',
    #         pickupdatetime=self.pickup,
    #         dropoffdatetime=self.dropoff,
    #         driverage='30',
    #         userip='175.156.244.174').parsed

    #     self.assertTrue(None != self.result.find('./Cars'))
    #     self.assertTrue(None != self.result.find('./Websites'))

    def test_create_session(self):
        """
        http://partners.api.skyscanner.net/apiservices/carhire/liveprices/v2/{market}/{currency}/{locale}/{pickupplace}/{dropoffplace}/{pickupdatetime}/{dropoffdatetime}/{driverage}?apiKey={apiKey}&userip={userip}
        YYYY-MM-DDThh:mm
        """
        carhire_service = CarHire(self.api_key)

        poll_url = carhire_service.create_session(
            market='UK',
            currency='GBP',
            locale='en-GB',
            pickupplace='LHR-sky',
            dropoffplace='LHR-sky',
            pickupdatetime=self.pickup,
            dropoffdatetime=self.dropoff,
            driverage='30',
            userip='175.156.244.174')

        self.assertTrue(poll_url)


class TestHotels(SkyScannerTestCase):

    def setUp(self):
        super(TestHotels, self).setUp()
        datetime_format = '%Y-%m-%d'
        checkin_datetime = datetime.now()
        checkout_datetime = checkin_datetime + timedelta(days=4)
        self.checkin = checkin_datetime.strftime(datetime_format)
        self.checkout = checkout_datetime.strftime(datetime_format)

    def test_location_autosuggest_json(self):
        hotels_service = Hotels(self.api_key, response_format='json')

        self.result = hotels_service.location_autosuggest(
            market='UK',
            currency='GBP',
            locale='en-GB',
            query='Kuala').parsed

        self.assertTrue('results' in self.result)
        self.assertTrue(len(self.result['results']) > 0)

    def test_location_autosuggest_xml(self):
        hotels_service = Hotels(self.api_key, response_format='xml')

        self.result = hotels_service.location_autosuggest(
            market='DE',
            currency='EUR',
            locale='de-DE',
            query='Berlin').parsed

        self.assertTrue(None != self.result.find('./Results'))
        self.assertTrue(
            len(self.result.findall('./Results/HotelResultDto')) > 0)

    def test_get_result_json(self):
        """
        http://partners.api.skyscanner.net/apiservices/carhire/liveprices/v2/{market}/{currency}/{locale}/{pickupplace}/{dropoffplace}/{pickupdatetime}/{dropoffdatetime}/{driverage}?apiKey={apiKey}&userip={userip}
        YYYY-MM-DDThh:mm
        """

        hotels_service = Hotels(self.api_key, response_format='json')
        self.result = hotels_service.get_result(
            market='UK',
            currency='GBP',
            locale='en-GB',
            entityid=27543923,
            checkindate=self.checkin,
            checkoutdate=self.checkout,
            guests=1,
            rooms=1).parsed

        self.assertTrue('hotels' in self.result)
        self.assertTrue(len(self.result['hotels']) > 0)

    def test_get_result_xml(self):
        """
        http://partners.api.skyscanner.net/apiservices/carhire/liveprices/v2/{market}/{currency}/{locale}/{pickupplace}/{dropoffplace}/{pickupdatetime}/{dropoffdatetime}/{driverage}?apiKey={apiKey}&userip={userip}
        YYYY-MM-DDThh:mm
        """

        hotels_service = Hotels(self.api_key, response_format='xml')
        self.result = hotels_service.get_result(
            market='DE',
            currency='EUR',
            locale='de-DE',
            entityid=27543923,
            checkindate=self.checkin,
            checkoutdate=self.checkout,
            guests=1,
            rooms=1).parsed

        self.assertTrue(None != self.result.find('./Hotels'))
        self.assertTrue(len(self.result.findall('./Hotels/HotelDto')) > 0)

    def test_create_session(self):
        """
        http://partners.api.skyscanner.net/apiservices/carhire/liveprices/v2/{market}/{currency}/{locale}/{pickupplace}/{dropoffplace}/{pickupdatetime}/{dropoffdatetime}/{driverage}?apiKey={apiKey}&userip={userip}
        YYYY-MM-DDThh:mm
        """
        hotels_service = Hotels(self.api_key)

        poll_url = hotels_service.create_session(
            market='UK',
            currency='GBP',
            locale='en-GB',
            entityid=27543923,
            checkindate=self.checkin,
            checkoutdate=self.checkout,
            guests=1,
            rooms=1)

        self.assertTrue(poll_url)


class TestFlights(SkyScannerTestCase):

    def setUp(self):
        super(TestFlights, self).setUp()
        datetime_format = '%Y-%m'
        outbound_datetime = datetime.now() + timedelta(days=7)
        inbound_datetime = outbound_datetime + timedelta(days=31)
        self.outbound = outbound_datetime.strftime(datetime_format)
        self.inbound = inbound_datetime.strftime(datetime_format)

        datetime_format = '%Y-%m-%d'
        inbound_datetime = outbound_datetime + timedelta(days=3)
        self.outbound_days = outbound_datetime.strftime(datetime_format)
        self.inbound_days = inbound_datetime.strftime(datetime_format)

    def test_get_cheapest_quotes_json(self):
        flights_cache_service = FlightsCache(
            self.api_key, response_format='json')
        self.result = flights_cache_service.get_cheapest_quotes(
            market='GB',
            currency='GBP',
            locale='en-GB',
            originplace='SIN',
            destinationplace='KUL',
            outbounddate=self.outbound,
            inbounddate=self.inbound).parsed

        self.assertTrue('Quotes' in self.result)
        self.assertTrue(len(self.result['Quotes']) > 0)

    def test_get_cheapest_quotes_xml(self):
        flights_cache_service = FlightsCache(
            self.api_key, response_format='xml')
        self.result = flights_cache_service.get_cheapest_quotes(
            market='DE',
            currency='EUR',
            locale='de-DE',
            originplace='TXL',
            destinationplace='LHR',
            outbounddate=self.outbound,
            inbounddate=self.inbound).parsed

        self.assertTrue(None != self.result.find('./Quotes'))
        self.assertTrue(len(self.result.findall('./Quotes/QuoteDto')) > 0)

    def test_get_cheapest_price_by_route_json(self):
        flights_cache_service = FlightsCache(
            self.api_key, response_format='json')
        self.result = flights_cache_service.get_cheapest_price_by_route(
            market='GB',
            currency='GBP',
            locale='en-GB',
            originplace='GB',
            destinationplace='DE',
            outbounddate=self.outbound,
            inbounddate=self.inbound).parsed

        self.assertTrue('Routes' in self.result)
        self.assertTrue(len(self.result['Routes']) > 0)

    def test_get_cheapest_price_by_route_xml(self):
        flights_cache_service = FlightsCache(
            self.api_key, response_format='xml')
        self.result = flights_cache_service.get_cheapest_price_by_route(
            market='DE',
            currency='EUR',
            locale='de-DE',
            originplace='DE',
            destinationplace='GB',
            outbounddate=self.outbound,
            inbounddate=self.inbound).parsed

        self.assertTrue(None != self.result.find('./Routes'))
        self.assertTrue(len(self.result.findall('./Routes/RouteDto')) > 0)

    def test_get_cheapest_price_by_date_json(self):
        flights_cache_service = FlightsCache(
            self.api_key, response_format='json')
        self.result = flights_cache_service.get_cheapest_price_by_date(
            market='GB',
            currency='GBP',
            locale='en-GB',
            originplace='SIN',
            destinationplace='KUL',
            outbounddate=self.outbound,
            inbounddate=self.inbound).parsed

        self.assertTrue('Quotes' in self.result)
        self.assertTrue(len(self.result['Quotes']) > 0)

    def test_get_cheapest_price_by_date_xml(self):
        flights_cache_service = FlightsCache(
            self.api_key, response_format='xml')
        self.result = flights_cache_service.get_cheapest_price_by_date(
            market='DE',
            currency='EUR',
            locale='de-DE',
            originplace='TXL',
            destinationplace='LHR',
            outbounddate=self.outbound,
            inbounddate=self.inbound).parsed

        self.assertTrue(None != self.result.find('./Quotes'))
        self.assertTrue(len(self.result.findall('./Quotes/QuoteDto')) > 0)

    def test_get_grid_prices_by_date_json(self):
        flights_cache_service = FlightsCache(
            self.api_key, response_format='json')
        self.result = flights_cache_service.get_grid_prices_by_date(
            market='GB',
            currency='GBP',
            locale='en-GB',
            originplace='SIN',
            destinationplace='KUL',
            outbounddate=self.outbound,
            inbounddate=self.inbound).parsed

        self.assertTrue('Dates' in self.result)
        self.assertTrue(len(self.result['Dates']) > 0)

    def test_get_grid_prices_by_date_xml(self):
        flights_cache_service = FlightsCache(
            self.api_key, response_format='xml')
        self.result = flights_cache_service.get_grid_prices_by_date(
            market='DE',
            currency='EUR',
            locale='de-DE',
            originplace='TXL',
            destinationplace='LHR',
            outbounddate=self.outbound,
            inbounddate=self.inbound).parsed

        self.assertTrue(None != self.result.find('./Dates'))
        self.assertTrue(len(self.result.findall('./Dates/ArrayOfCellDto')) > 0)

    def test_create_session(self):
        flights_service = Flights(self.api_key)
        poll_url = flights_service.create_session(
            country='UK',
            currency='GBP',
            locale='en-GB',
            originplace='SIN-sky',
            destinationplace='KUL-sky',
            outbounddate=self.outbound_days,
            inbounddate=self.inbound_days,
            adults=1)

        self.assertTrue(poll_url)

    def test_get_additional_params(self):
        flights_service = Flights(self.api_key)

        additional_params = flights_service.get_additional_params(
            stops=0,
            country='US',
            currency='USD',
            locale='en-US',
            originplace='DTW-sky',
            destinationplace='PEK-sky',
            outbounddate=self.outbound,
            inbounddate=self.inbound)
        self.assertEqual(additional_params, {'stops': 0})
    
    # def test_poll_session(self):
    #     flights_service = Flights(self.api_key)

    #     poll_url = flights_service.create_session(
    #         country='UK',
    #         currency='GBP',
    #         locale='en-GB',
    #         originplace='SIN-sky',
    #         destinationplace='KUL-sky',
    #         outbounddate='2015-05-28',
    #         inbounddate='2015-05-31',
    #         adults=1)

    #     result = flights_service.poll_session(poll_url, sorttype='carrier')

    #     self.assertTrue(len(result['Itineraries']) > 0)

    #     pass

    # def test_request_booking_details(self):
    #     flights_service = Flights(self.api_key)

    #     poll_url = flights_service.create_session(
    #         country='UK',
    #         currency='GBP',
    #         locale='en-GB',
    #         originplace='SIN-sky',
    #         destinationplace='KUL-sky',
    #         outbounddate='2015-05-28',
    #         inbounddate='2015-05-31',
    #         adults=1)

    #     flights_results = flights_service.poll_session(poll_url, sorttype='carrier')

    #     print(flights_results)

    #     itinerary = flights_results['Itineraries'][0]

    #     result = flights_service.request_booking_details(poll_url, outboundlegid=itinerary['OutboundLegId'],
    #                                                      inboundlegid=itinerary['InboundLegId'])

    #     print(result)

    #     pass

    def test_get_result_json(self):
        flights_service = Flights(self.api_key, response_format='json')
        self.result = flights_service.get_result(
            errors=GRACEFUL,
            country='UK',
            currency='GBP',
            locale='en-GB',
            originplace='SIN-sky',
            destinationplace='KUL-sky',
            stops=1,
            outbounddate=self.outbound_days,
            inbounddate=self.inbound_days,
            adults=1).parsed

        self.assertTrue(None != self.result)
        self.assertTrue('Itineraries' in self.result)
        self.assertTrue(len(self.result['Itineraries']) > 0)

    def test_get_result_xml(self):
        flights_service = Flights(self.api_key, response_format='xml')
        self.result = flights_service.get_result(
            errors=GRACEFUL,
            country='UK',
            currency='GBP',
            locale='en-GB',
            originplace='SIN-sky',
            destinationplace='KUL-sky',
            outbounddate=self.outbound_days,
            inbounddate=self.inbound_days,
            adults=1).parsed

        self.assertTrue(None != self.result)
        self.assertTrue(None != self.result.find('./Itineraries'))
        self.assertTrue(
            len(self.result.findall('./Itineraries/ItineraryApiDto')) > 0)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    # TODO: put package requirements here
    'requests'
]
extras_requirements = {
    'Faster XML processing':  ["lxml"]
}
test_requirements = [
    # TODO: put package test requirements here
    'requests'
]

setup(
    name='skyscanner',
    version='1.1.2',
    description="Skyscanner Python SDK",
    long_description=readme + '\n\n' + history,
    author="Ardy Dedase",
    author_email='ardy.dedase@skyscanner.net',
    url='https://github.com/Skyscanner/skyscanner-python-sdk',
    packages=[
        'skyscanner',
    ],
    package_dir={'skyscanner':
                 'skyscanner'},
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_requirements,
    license="Apache",
    zip_safe=False,
    keywords='skyscanner travel api business sdk flights hotels',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
